--[[ CombatTime ]]

if not FixR then return end

local DefaultO = {
  ["framePoint"] = "CENTER";
  ["frameRelativeTo"] = "UIParent";
  ["frameRelativePoint"] = "CENTER";
  ["frameOffsetX"] = 0;
  ["frameOffsetY"] = 0;
  ["showCombatLogging"] = false;
  ["continueEncounterWhenDead"] = true;
}
local misc = {
  ["desc"] = "Display combat length";
  ["alpha"] = true;
  ["GUI"] = {
    [1] = {
      ["type"] = "Boolean";
      ["opt"] = "showCombatLogging";
      ["desc"] = "display combat logging state";
    };
    [2] = {
      ["type"] = "Boolean";
      ["opt"] = "continueEncounterWhenDead";
      ["desc"] = "continue to count when your character died but the encounter is still in progress";
    };
    [3] = {
      ["type"] = "Button";
      ["opt"] = "toggle moving";
      ["func"] = function(self) self:SetOption("move"); end;
    };
    [4] = {
      ["type"] = "Button";
      ["opt"] = "reset position";
      ["func"] = function(self) self:SetOption("reset"); end;
    };
  };
}

local M = FixR:NewModule("CombatTime", DefaultO, misc)

local frame = nil
local frameEvents = {};
local moving = false
local absorbAmount = 0

local function initFrame()
  frame = CreateFrame("Frame", "FixR_CombatTimeFrame", UIParent)
  frame:SetPoint(M.O["framePoint"], M.O["frameRelativeTo"], M.O["frameRelativePoint"], M.O["frameOffsetX"], M.O["frameOffsetY"])
  frame:SetFrameStrata("HIGH")
  frame:SetSize(220, 40)
  frame:SetScript("OnEvent", function(self, event, ...)
    frameEvents[event](self, ...)
  end);
  frame:SetScript("OnDragStart", function(self) self:StartMoving(); end);
  frame:SetScript("OnDragStop", function(self)
    self:StopMovingOrSizing()
    point, relativeTo, relativePoint, xOfs, yOfs = self:GetPoint()
    M.O["framePoint"] = point or "CENTER"
    M.O["frameRelativeTo"] = relativeTo or "UIParent"
    M.O["frameRelativePoint"] = relativePoint or "CENTER"
    M.O["frameOffsetX"] = FixR:Round(xOfs, 4)
    M.O["frameOffsetY"] = FixR:Round(yOfs, 4)
  end);
  frame:SetScript("OnUpdate", function(self, elapsed)
    self.OnUpdate(self, elapsed)
  end);
  frame.timeSinceLastUpdate = 0
  frame.updateNeeded = false
  frame.inCombat = false
  frame.encounterInProgress = false
  frame.secondsInCombat = 0
  
  frame.combatLogging = LoggingCombat() and true or false
  
  frame.text = frame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
  frame.text:SetPoint("RIGHT")
  frame.text:SetText("CombatTime")
  frame.text:SetTextColor(1, 1, 1, 1)
  
  frame.OnUpdate = function(self, elapsed)
    self.timeSinceLastUpdate = self.timeSinceLastUpdate + elapsed
    while (self.encounterInProgress and self.timeSinceLastUpdate > 1) do --in seconds
      self.secondsInCombat = self.secondsInCombat + 1
      self.updateNeeded = true
      if self.inCombat or M.O["continueEncounterWhenDead"] and IsEncounterInProgress() then
        self.encounterInProgress = true
      else
        self.encounterInProgress = false
      end
      
      self.timeSinceLastUpdate = self.timeSinceLastUpdate - 1
    end
    
    if self.updateNeeded == true then
      self.updateNeeded = false
      self.updateDisplay()
    end
    
  end
  frame.updateDisplay = function()
    local l = frame.secondsInCombat
    local d = math.floor(l / 86400)
    l = l - 86400 * d
    local h = math.floor(l / 3600)
    l = l - 3600 * h
    local m = math.floor(l / 60)
    l = l - 60 * m
    local s = l
    
    local t
    
    if d > 0 then
      if m < 10 then
        m = "0"..m
      end
      if s < 10 then
        s = "0"..s
      end
      t = string.format("%dd %d:%s:%s", d, h, m, s)
    elseif h > 0 then
      if m < 10 then
        m = "0"..m
      end
      if s < 10 then
        s = "0"..s
      end
      t = string.format("%d:%s:%s", h, m, s)
    elseif m > 0 then
      if s < 10 then
        s = "0"..s
      end
      t = string.format("%d:%s", m, s)
    else
      t = s
    end
    if M.O["showCombatLogging"] then
      if frame.combatLogging then
        t = t.." |cff00ff00ON"
      else
        t = t.." |cffff0000OFF"
      end
    end
    frame.text:SetText(t)
  end
  frame.combatLoggingEvent = function(msg)
    --isLogging = LoggingCombat([newState])
    frame.combatLogging = LoggingCombat() and true or false
    frame.updateDisplay()
  end
  hooksecurefunc(SlashCmdList, "COMBATLOG", frame.combatLoggingEvent)
end

--[[
function frameEvents:UNIT_ABSORB_AMOUNT_CHANGED(unit)
  if unit == "player" then
    absorbAmount = UnitGetTotalAbsorbs("player")
    self.updateDisplay()
  end
end
--]]
function frameEvents:PLAYER_REGEN_DISABLED(...)
  if self.encounterInProgress == false then
    self.encounterInProgress = true
    self.secondsInCombat = 0
    self.timeSinceLastUpdate = 0
  end
  self.inCombat = true
  self.updateDisplay()
end
function frameEvents:PLAYER_REGEN_ENABLED(...)
  self.inCombat = false
  self.updateDisplay()
end



function M:Load()
  if not frame then
    initFrame()
  end
  for k, v in pairs(frameEvents) do
    frame:RegisterEvent(k);
  end
  frame:Show()
end

function M:Unload()
  if frame then
    for k, v in pairs(frameEvents) do
      frame:UnregisterEvent(k);
    end
    frame:Hide()
  end
end

function M:SetOption(arg1, arg2)
  if arg1 == "move" then
    if moving then
      moving = false
      --frame.texture:SetTexture(0.67, 0.67, 1, 0.01) -- cannot :Hide() in combat, so make it nearly transparent
      frame:SetMovable(false) --click-through
      frame:EnableMouse(false)
      frame:RegisterForDrag("")
    else
      moving = true
      --frame.texture:SetTexture(0.67, 0.67, 1, 1) --opaque
      frame:SetMovable(true);
      frame:EnableMouse(true);
      frame:RegisterForDrag("LeftButton");
    end
    FixR:Echo("CombatTime: now "..(moving and "moving" or "fixed"))
  elseif arg1 == "reset" then
    M.O["framePoint"] = DefaultO["framePoint"]
    M.O["frameRelativeTo"] = DefaultO["frameRelativeTo"]
    M.O["frameRelativePoint"] = DefaultO["frameRelativePoint"]
    M.O["frameOffsetX"] = DefaultO["frameOffsetX"]
    M.O["frameOffsetY"] = DefaultO["frameOffsetY"]
    
    frame:ClearAllPoints()
    frame:SetPoint(M.O["framePoint"], M.O["frameRelativeTo"], M.O["frameRelativePoint"], M.O["frameOffsetX"], M.O["frameOffsetY"])
  elseif arg1 == "showCombatLogging" then
    local bln, updateGUI = nil, true
    if arg2 == 1 then
      bln = true
      updateGUI = false
    elseif arg1 == 0 then
      bln = false
      updateGUI = false
    else --toggle
      bln = not (M.O["showCombatLogging"] or false)
    end
    M.O["showCombatLogging"] = bln
    FixR:Echo("CombatTime: showCombatLogging: "..(bln and "|cffaaffaaOn" or "|cffff8888Off"))
    if frame then
      frame.updateDisplay()
    end
    if updateGUI then
      FixR:SetModuleGUIOption(M, 1, bln)
    end
  elseif arg1 == "continueEncounterWhenDead" then
    local bln, updateGUI = nil, true
    if arg2 == 1 then
      bln = true
      updateGUI = false
    elseif arg1 == 0 then
      bln = false
      updateGUI = false
    else --toggle
      bln = not (M.O["continueEncounterWhenDead"] or false)
    end
    M.O["continueEncounterWhenDead"] = bln
    FixR:Echo("CombatTime: continueEncounterWhenDead: "..(bln and "|cffaaffaaOn" or "|cffff8888Off"))
    if updateGUI then
      FixR:SetModuleGUIOption(M, 2, bln)
    end
  end
end

function M:ShowOptions()
end
